%%%%% MATLAB ROUTINE FOR  BY FLAT AUTO PROBIT MODEL         %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [z, mu, beta] = Auto_probit_car_cv(A, label, tau2, sigma_beta, sigma_mu, T, Fold)

%% Parameter setting
n = length(label);
% Diagnol matrix D
d = sum(A);
D = diag(d);
B = D^(0.5)*A*D^(0.5);
lambda = eig(B); 
lambda = sort( lambda(imag(lambda) == 0) );
% Boundary for beta
beta_min = 0; 
beta_max = 1/lambda(length(lambda));
m = 0.5*beta_min+0.5*beta_max;
% Boundary for mu's
u = 4;
% Boundary for z's
v = 30; 


%% Initializing
z = zeros(n, 1, Fold);
mu = zeros(n, 1, Fold);
beta = ones(T, 1, Fold);


for fold = 1 : Fold
    if fold < Fold
        % Test set
        test = 1 + (fold - 1) * floor(n/Fold) : fold * floor(n/Fold);
        % Training set
        train = 1 : n;
        train(test) = zeros(size(test));
        train = train((train>0));
        
        %%%%% MCMC %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for t = 2 : T
    
            %% Step1: Update z individually
            z(test, t) = z(test, t-1);
            for i = 1 : length(test)
                z(test(i), t) = randn + mu(test(i), t-1) + beta(t-1)*( B(test(i), :) * (z(:, t-1)-mu(:,t-1)) );
            end;
            
            z(train, t) = z(train, t-1);
            for i = 1 : length(train)
                if label(train(i)) == 1
                    % randraw( 'normaltrunc', [a, b, mu, sigma], SampleSize )
                    z(train(i), t) = randraw('normaltrunc', [0, v, mu(train(i), t-1) + beta(t-1)*( B(train(i), :) * (z(:, t-1) - mu(:,t-1)) ), 1], 1);
           
                elseif label(train(i)) == -1
                    z(train(i), t) = randraw('normaltrunc', [-v, 0, mu(train(i), t-1) + beta(t-1)*( B(train(i), :) * (z(:, t-1) - mu(:,t-1)) ), 1], 1);
                end;
            end;
                
            %% Step 2: Update mu by Metropolis_Hastings Alg
            mu(:, t) = mu(:, t-1);
            for i = 1 : n
                old_mu = mu(:, t);
                %% update the ith element of mu in the *current* iteration
                mu(i, t) = randraw('normaltrunc', [-u, u, old_mu(i), sigma_mu], 1);
         
                comp1 = 2*(z(i, t) - mu(i, t))*(mu(i, t) - old_mu(i));
                comp2 = (mu(i, t) - old_mu(i))*(mu(i, t) - old_mu(i));
                comp3 = -2*beta(t-1)*(mu(i, t) - old_mu(i))*B(i, :)*(z(:, t) - mu(:, t));
                rp = exp( 0.5*(comp1 + comp2 + comp3) );
                % rprior = normpdf( mu(i, t), sum(mu((B(:, i)>0), t-1))/degree(i), sqrt( tau2/degree(i) ) )...
                  %      / normpdf(old_mu(i), sum(mu((B(:, i)>0), t-1))/degree(i), sqrt( tau2/degree(i) ));
                
               rprior = normpdf( mu(i, t), -A(i, :)*mu(:, t)/d(i), sqrt( tau2/d(i) ))...
                         / normpdf( old_mu(i), -A(i, :)*mu(:, t)/d(i), sqrt(tau2/d(i)) );


                rprop = (normcdf(u, old_mu(i), sigma_mu) - normcdf(-u, old_mu(i), sigma_mu))...
                    /(normcdf(u, mu(i, t), sigma_mu) - normcdf(-u, mu(i, t), sigma_mu));
        
                accept_mu = rp * rprior * rprop;
                if accept_mu < rand
                    mu(i, t) = mu(i, t-1);
                end;
            end;
            
            %% Step 3: Update beta by Metropolis_hastings alg with Truncated Normal proposal
            new_beta = randraw('normaltrunc', [beta_min, beta_max, beta(t - 1), sigma_beta]);
    
            %normcdf(X,mu,sigma) 
            rq = (normcdf( beta_max, beta(t - 1), sigma_beta ) - normcdf( 0, beta(t - 1), sigma_beta ))...
                 /(normcdf( beta_max, new_beta, sigma_beta ) - normcdf( 0, new_beta, sigma_beta ));
            rp = prod(1 - new_beta*lambda)/prod(1 - beta(t-1)*lambda)  *...
                  exp( -0.5*(z(:, t) - mu(:, t))'*((beta(t-1) - new_beta)*B)*(z(:, t) - mu(:, t)) ) ...
                 * normpdf( new_beta, m, sigma_beta )/normpdf( beta(t-1), m, sigma_beta ) ;
            accept_beta = rq* rp;
            if accept_beta >= rand
                beta(t) = new_beta;
            else
                beta(t) = beta(t-1);
            end;
            
        end;
        Z(:, :, fold) = z;
        MU(:, :, fold) = mu;
        BETA(:, fold) = beta';
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
    elseif fold == Fold
         test = 1 + (fold - 1) * floor(n/Fold) : n;
         train = 1 : n;
         train(test) = zeros(size(test));
         train = train((train>0));
         
         %%%%% MCMC %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        for t = 2 : T
    
            %% Step1: Update z individually
            z(test, t) = z(test, t-1);
            for i = 1 : length(test)
                z(test(i), t) = randn + mu(test(i), t-1) + beta(t-1)*( B(test(i), :) * (z(:, t-1) - mu(:,t-1)) );
            end;
            
            z(train, t) = z(train, t-1);
            for i = 1 : length(train)
                if label(train(i)) == 1
                    % randraw( 'normaltrunc', [a, b, mu, sigma], SampleSize )
                    z(train(i), t) = randraw('normaltrunc', [0, 3, mu(train(i), t-1) + beta(t-1)*( B(train(i), :) * (z(:, t-1) - mu(:,t-1)) ), 1], 1);
           
                elseif label(train(i)) == -1
                    z(train(i), t) = randraw('normaltrunc', [-3, 0, mu(train(i), t-1) + beta(t-1)*( B(train(i), :) * (z(:, t-1) - mu(:, t-1)) ), 1], 1);
                end;
            end;
    
    
            %% Step 2: Update mu by Metropolis_Hastings Alg
            mu(:, t) = mu(:, t-1);
            for i = 1 : n
                old_mu = mu(:, t);
        
                %% update the ith element of mu in the *current* iteration
                mu(i, t) = randraw('normaltrunc', [-4, 4, old_mu(i), sigma_mu], 1);
         
                comp1 = 2*(z(i, t) - mu(i, t))*(mu(i, t) - old_mu(i));
                comp2 = (mu(i, t) - old_mu(i))*(mu(i, t) - old_mu(i));
                comp3 = -2*beta(t-1)*(mu(i, t) - old_mu(i))*B(i, :)*(z(:, t) - mu(:, t));
                rp = exp( 0.5*(comp1 + comp2 + comp3) );
                % rprior = normpdf( mu(i, t), sum(mu((B(:, i)>0), t-1))/degree(i), sqrt( tau2/degree(i) ) )...
                  %       / normpdf(old_mu(i), sum(mu((B(:, i)>0), t-1))/degree(i), sqrt( tau2/degree(i) ));
                
	        rprior = normpdf( mu(i, t), -A(i, :)*mu(:, t)/d(i), sqrt( tau2/d(i) ))...
		       / normpdf( old_mu(i), -A(i, :)*mu(:, t)/d(i), sqrt(tau2/d(i)) );

                rprop = (normcdf(4, old_mu(i), sigma_mu) - normcdf(-4, old_mu(i), sigma_mu))/(normcdf(4, mu(i, t), sigma_mu) - normcdf(-4, mu(i, t), sigma_mu));
                accept_mu = rp * rprior * rprop;
                if accept_mu < rand
                    mu(i, t) = mu(i, t-1);
                end;
            end;
            
            %% Step 3: Update beta by Metropolis_hastings alg with Truncated Normal proposal
            new_beta = randraw('normaltrunc', [beta_min, beta_max, beta(t - 1), sigma_beta]);
    
            %normcdf(X,mu,sigma) 
            rq = (normcdf( beta_max, beta(t - 1), sigma_beta ) - normcdf( 0, beta(t - 1), sigma_beta ))...
                 /(normcdf( beta_max, new_beta, sigma_beta ) - normcdf( 0, new_beta, sigma_beta ));
    
            rp = prod(1 - new_beta*lambda)/prod(1 - beta(t-1)*lambda)  *...
                  exp( -0.5*(z(:, t) - mu(:, t))'*((beta(t-1) - new_beta)*B)*(z(:, t) - mu(:, t)) ) ...
                 * normpdf( new_beta, m, sigma_beta )/normpdf( beta(t-1), m, sigma_beta ) ;
    
            accept_beta = rq* rp;

            if accept_beta >= rand
                beta(t) = new_beta;
            else
                beta(t) = beta(t-1);
            end;
        end;
        Z(:, :, fold) = z;
        MU(:, :, fold) = mu;
        BETA(:, fold) = beta';
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    end;
end;
