%%%%% MATLAB ROUTINE FOR FLAT AUTO PROBIT MODEL                       %%%%%
%%%%% Z ~ MVN( \MU, (I - D^.5*AD^.5)^(-1) )
%%%%%   with constant \tau2 and labeling probability g
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



function [z, mu, beta, g] = Auto_probit_car_g(A, label, tau2, sigma_beta, sigma_mu, T)


%% Parameter setting
n = length(label);
% Diagnol matrix D
d = sum(A);
D = diag(d);
B = D^(0.5)*A*D^(0.5);
lambda = eig(B); 
lambda = sort( lambda(imag(lambda) == 0) );
% Boundary for beta
beta_min = 0; 
beta_max = 1/lambda(length(lambda));
m = 0.5*beta_min+0.5*beta_max;
% Boundary for mu's
u = 4;
% Boundary for z's
v = 30; 


%% Initializing
mu = zeros(n, 1);
z = zeros(n, 1);
beta = ones(T, 1);
g = zeros(T, 1);
g(1) = 0.5;


%%%%% MCMC %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for t = 2 : T
    %%%%%
    %% Step1: Update z individually
    for i = 1 : n
        if label(i) == 1
           % randraw( 'normaltrunc', [a, b, mu, sigma], SampleSize )
           z(i, t) = randraw('normaltrunc', [0, v, mu(i, t-1) + beta(t-1)*( B(i, :) * (z(:, t-1) - mu(:, t-1)) ), 1], 1);
           
           elseif label(i) == -1
               if g(t-1, 1) < rand
                   z(i, t) = randraw('normaltrunc', [-v, 0, mu(i, t-1) + beta(t-1)*( B(i, :) * (z(:, t-1) - mu(:, t-1)) ), 1], 1);
               else
                   z(i, t) = randraw('normaltrunc', [0, v, mu(i, t-1) + beta(t-1)*( B(i, :) * (z(:, t-1) - mu(:, t-1)) ), 1], 1);
               end;
        end;
    end;
           
    %% Step 2: Update mu by Metropolis-Hasting Alg
    mu(:, t) = mu(:, t-1);
    for i = 1 : n
        old_mu = mu(:, t);
        %% update the ith element of mu in the *current* iteration
        mu(i, t) = randraw('normaltrunc', [-u, u, old_mu(i), sigma_mu], 1);
                 
        comp1 = 2*(z(i, t) - mu(i, t))*(mu(i, t) - old_mu(i));
        comp2 = (mu(i, t) - old_mu(i))*(mu(i, t) - old_mu(i));
        comp3 = -2*beta(t-1)*(mu(i, t) - old_mu(i))*B(i, :)*(z(:, t) - mu(:, t));
        rp = exp( 0.5*(comp1 + comp2 + comp3) );
       
        rprior = normpdf( mu(i, t), -A(i, :)*mu(:, t)/d(i), sqrt( tau2/d(i) ))...
                 / normpdf( old_mu(i), -A(i, :)*mu(:, t)/d(i), sqrt(tau2/d(i)) );

        rprop = (normcdf(u, old_mu(i), sigma_mu) - normcdf(-u, old_mu(i), sigma_mu))/...
                (normcdf(u, mu(i, t), sigma_mu) - normcdf(-u, mu(i, t), sigma_mu));
        
        accept_mu = rp * rprior * rprop;
        if accept_mu < rand
            mu(i, t) = mu(i, t-1);
        end;
    end;
        
    %% Step 3: Update beta by Metropolis_hastings alg with Truncated Normal proposal
    new_beta = randraw('normaltrunc', [beta_min, beta_max, beta(t - 1), sigma_beta]);
    %normcdf(X,mu,sigma) 
    rq = (normcdf( beta_max, beta(t - 1), sigma_beta ) - normcdf( beta_min, beta(t - 1), sigma_beta ))...
          /(normcdf( beta_max, new_beta, sigma_beta ) - normcdf( beta_min, new_beta, sigma_beta ));
    rp = prod(1 - new_beta*lambda)/prod(1 - beta(t-1)*lambda)  *...
          exp( -0.5*(z(:, t) - mu(:, t))'*((beta(t-1) - new_beta)*B)*(z(:, t) - mu(:, t)) ) ...
          * normpdf( new_beta, m, sigma_beta )/normpdf( beta(t-1), m, sigma_beta ) ;
    accept_beta = rq* rp;
    if accept_beta >= rand
        beta(t) = new_beta;
    else
        beta(t) = beta(t-1);
    end;
        
    
    %% Step 4: Update g by Gibbs sampling
    g(t, 1) = randraw('beta', [0, 1, sum((z(:, t)>=0) + (label==-1) == 2)+1, sum((z(:, t)>=0) + (label==1) == 2)+1  ], 1);
    
end;
