%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Matlab routine for computing True Positive (TP), False Positive (FP),
%%%        True Negative (TN) and False Negative (FN)
%%%  
%%% Inputs:   threshold := threshold applied to f, f >= threshold,
%%%                              prediction = 1; ow, prediction = -1, scalar
%%%           f         := probabilities produced from a certain method, vector
%%%           Y         := true labels, vector
%%% Outputs:  TP        := True Positive, scalar
%%%           FP        := False Positive, scalar
%%%           TN        := True Negative, scalar
%%%           FN        := False Negative, scalar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [TP, FP, TN, FN] = count(threshold, f, Y);
         TN = 0;
         FN = 0;
         TP = 0;
         FP = 0;

        for i = 1 : length(f)
            if f(i) >= threshold & Y(i) == 1
               TP = TP + 1;
            elseif f(i) >= threshold & Y(i) == -1
               FP = FP + 1;
            elseif f(i) < threshold & Y(i) == 1
               FN = FN + 1;
            elseif f(i) < threshold & Y(i) == -1
               TN = TN + 1;
            end;
        end;
